#-*- coding: utf-8 -*-
from Components.ConfigList import ConfigListScreen
from Screens.Screen import Screen
from Tools.Directories import fileExists, resolveFilename, SCOPE_PLUGINS
from Components.ActionMap import NumberActionMap, ActionMap
from Components.Button import Button
from Components.Label import Label
from Screens.MessageBox import MessageBox
from Components.ScrollLabel import ScrollLabel
from Tools.LoadPixmap import LoadPixmap
from Components.config import config, ConfigSelection, getConfigListEntry, ConfigText, ConfigDirectory, ConfigYesNo, ConfigSelection, ConfigSubsection, ConfigPIN, configfile, ConfigInteger, NoSave, ConfigNothing
from Components.UsageConfig import preferredTimerPath, preferredInstantRecordPath
from Tools.Directories import resolveFilename, SCOPE_HDD
from enigma import eTimer
import re
import os
import shutil

# our custom classes
from SkyMainFunctions import getPluginPath, getCurrentTimestamp
from SkyGetTvGuide import SkyGetTvGuide
from SkyRunAutocheck import SkyRunAutocheck
from SkySelectAnytimeFolder import SkySelectAnytimeFolder
from SkySelectSkydbFolder import SkySelectSkydbFolder
from SkySql import *


pluginTitle = "Sky Anytime"
pluginVersion = "v0.8.2"
pluginSubVersion = "Beta"
pluginName = pluginTitle + " " + pluginVersion + " " + pluginSubVersion
contentSize = 20

pluginfolder = getPluginPath()

defaultmoviefolder = resolveFilename(SCOPE_HDD)
anytimefolder = defaultmoviefolder  #+ "SkyAnytime/"

### start config
config.plugins.skyanytime = ConfigSubsection()
config.plugins.skyanytime.refreshtimer = ConfigInteger(0) # deprecated
config.plugins.skyanytime.auto_recordtimer_entries = ConfigYesNo(default = False)

config.plugins.skyanytime.pluginname = ConfigText(pluginName)
config.plugins.skyanytime.contentsize = ConfigInteger(20)
config.plugins.skyanytime.timerdays_allowed = ConfigSelection(
			default = "['all']",
			choices = [
				("['all']", _("jeden Wochentag")),
				("['Mon','Tue','Wed','Thu','Fri']", _("Montag bis Freitag")),
				("['Sun','Sat']", _("Samstag und Sonntag")),
				("['Sun','Fri','Sat']", _("Freitag, Samstag und Sonntag"))])
config.plugins.skyanytime.fromtime = ConfigInteger(00, (00,23))
config.plugins.skyanytime.totime = ConfigInteger(23, (00,23))
config.plugins.skyanytime.refreshtimer_interval = ConfigSelection(
			default = "no",
			choices = [
				("no", _("nie")),
				("5", _("alle 5 Minuten (nur zum Testen)")),
				("15", _("alle 15 Minuten (nur zum Testen)")),
				("60", _("jede Stunde")),
				("180", _("alle 3 Stunden")),
				("360", _("alle 6 Stunden (empfohlen)")),
				("720", _("alle 12 Stunden")),
				("1440", _("einmal am Tag"))])
config.plugins.skyanytime.anytime_skin = ConfigSelection(
			default = "original",
			choices = [
				("elgato", _("elgato")),
				("original", _("original"))])
# store our last check-timestamp. needed for our wakeMeUp function (see below at the end of this script)
config.plugins.skyanytime.lastchecked = ConfigInteger(0)
config.plugins.skyanytime.wakeup = ConfigYesNo(default = False)

config.plugins.skyanytime.logfile_reset = NoSave(ConfigNothing())
config.plugins.skyanytime.added_reset = NoSave(ConfigNothing())

config.plugins.skyanytime.anytimefolder = ConfigText(default=anytimefolder)
config.plugins.skyanytime.fake_entry = NoSave(ConfigNothing())

#config.plugins.skyanytime.skydb = ConfigText(default=defaultmoviefolder + "skydb.db")
config.plugins.skyanytime.skydb = ConfigText(default=pluginfolder + "skydb.db")
config.plugins.skyanytime.skydb_fake_entry = NoSave(ConfigNothing())

config.plugins.skyanytime.silent_timer_mode = ConfigYesNo(default = False)
config.plugins.skyanytime.timer_mode = ConfigSelection(default = "0", choices = [("0", _("Aufnahme")),("1", _("Erinnerung"))])
config.plugins.skyanytime.msgtimeout = ConfigInteger(3)
### end config


# skysettings --> SkyAnytimeSettings
class SkyAnytimeSettings(Screen, ConfigListScreen):
	
	def __init__(self, session, firstRun=False):
		self.session = session
		
		path = "%s/skins/%s/screen_settings.xml" % (getPluginPath(), config.plugins.skyanytime.anytime_skin.value)
		with open(path, "r") as f:
			self.skin = f.read()
			f.close()	
		
		Screen.__init__(self, session)
		
		self.createConfigList()
		ConfigListScreen.__init__(self, self.configlist, session = self.session)
		
		
		self["actions"]  = ActionMap(["OkCancelActions", "ShortcutActions", "EPGSelectActions", "WizardActions", "ColorActions", "NumberActions", "MenuActions", "MoviePlayerActions"], {
			"ok" : self.keyOK,
			"cancel" : self.keyCancel,
			"nextBouquet" : self.log_up,
			"prevBouquet" : self.log_down,
			"blue" : self.askAddRecordimerNow,
			"menu" : self.readLog,
			"yellow" : self.readAdded,
			"green" : self.saveSettings, #self.askAddRecordimerNow,
			"red" : self.askCleanUpDatabase,
			"info" : self.askUpdateDatabase
		}, -1)

		
			#"green" : self.saveSettings,
			#"red" : self.cancelSave
		
		self['title'] = Label(pluginName)
		self['head'] = Label("Einstellungen")
		self["log"] = ScrollLabel()
		
		self.sky_log_path = "/usr/lib/enigma2/python/Plugins/Extensions/skyanytime/sky_log"
		self.sky_added_path = "/usr/lib/enigma2/python/Plugins/Extensions/skyanytime/sky_added"
		
		self.onLayoutFinish.append(self.readLog)
		
		
	def createConfigList(self):
		self.configlist = []
		self.configlist.append(getConfigListEntry("Datenbank aktualisieren:", config.plugins.skyanytime.refreshtimer_interval))
		self.configlist.append(getConfigListEntry("Hinzufügen von Automatischen Timern:", config.plugins.skyanytime.auto_recordtimer_entries ))
		# choose recordtimer mode
		self.configlist.append(getConfigListEntry("Timermodus:", config.plugins.skyanytime.timer_mode))
		self.configlist.append(getConfigListEntry("Timereinträge erlaubt für:", config.plugins.skyanytime.timerdays_allowed))
		self.configlist.append(getConfigListEntry("Zeitspanne von:", config.plugins.skyanytime.fromtime))
		self.configlist.append(getConfigListEntry("Zeitspanne bis:", config.plugins.skyanytime.totime))
		self.configlist.append(getConfigListEntry("Deep Standby beenden für Timerupdates:", config.plugins.skyanytime.wakeup))

		self.get_anytimefolder = getConfigListEntry("Anytime-Ordner: " + config.plugins.skyanytime.anytimefolder.value, config.plugins.skyanytime.fake_entry)
		self.configlist.append(self.get_anytimefolder)
		
		self.configlist.append(getConfigListEntry("Skinauswahl:", config.plugins.skyanytime.anytime_skin))
		self.configlist.append(getConfigListEntry("Keine Timermeldungen anzeigen:", config.plugins.skyanytime.silent_timer_mode))
		self.reset_logfile = getConfigListEntry("Logdatei zurücksetzen", config.plugins.skyanytime.logfile_reset)
		self.configlist.append(self.reset_logfile)
		
		self.reset_added = getConfigListEntry("Timerliste zurücksetzen", config.plugins.skyanytime.added_reset)
		self.configlist.append(self.reset_added)
		
		self.get_skydbfolder = getConfigListEntry("Datenbank: " + config.plugins.skyanytime.skydb.value, config.plugins.skyanytime.skydb_fake_entry)
		self.configlist.append(self.get_skydbfolder)
		
		
	def readLog(self):
		
		try:
			sql.cur.execute('SELECT SQLITE_VERSION()')
		except:
			try:
				sql.connect()
			except:
				return
		
		if fileExists(self.sky_log_path):
			# read logfile in reversed order so the newest entry is shown on top of the list
			read_log = open(self.sky_log_path, "r")
			text = ""
			for rawData in reversed(read_log.readlines()):
				data = re.findall('"(.*?)"', rawData, re.S)
				if data:
					dump = "%s\n" % data[0]
					text += dump
			read_log.close()
			self["log"].setText(text)
		
	def log_up(self):
		self["log"].pageUp()

	def log_down(self):
		self["log"].pageDown()		
	
	def readAdded(self):
		text = ""
		try:
			for recordedfile in sql.readAdded():
				text += "{0}\n".format(recordedfile[0])
			self["log"].setText(text)
		except:
			pass
		
	
	def keyOK(self):
		if self["config"].getCurrent() == self.get_anytimefolder:
			self.session.openWithCallback(self.gotAnytimeFolder, SkySelectAnytimeFolder, config.plugins.skyanytime.anytimefolder.value)

		elif self["config"].getCurrent() == self.get_skydbfolder:
			if config.plugins.skyanytime.refreshtimer_interval.value is not "no":
				message = self.session.open(MessageBox, _("Achtung!\nBitte zuerst die automatische Datenbankaktualisierung beenden und das Plugin neu starten. Danach kann der Speicherort der Datenbank geändert werden."), MessageBox.TYPE_INFO, timeout=-1)
				return
			self.session.openWithCallback(self.gotSkydbFolder, SkySelectSkydbFolder, config.plugins.skyanytime.skydb.value)

		elif self["config"].getCurrent() == self.reset_logfile:
			mymsg = "{0}\nSoll die Logdatei jetzt gelöscht werden?".format(pluginName)
			self.session.openWithCallback(self.resetLogfile,  MessageBox, _(mymsg), MessageBox.TYPE_YESNO, timeout=-1, default=True)
			
		elif self["config"].getCurrent() == self.reset_added:
			mymsg = "{0}\nSoll die Timerliste jetzt gelöscht werden?\nAufnahmetimer werden dadurch nicht entfernt. Lediglich die interne Liste wird zurückgesetzt.".format(pluginName)
			self.session.openWithCallback(self.resetAdded,  MessageBox, _(mymsg), MessageBox.TYPE_YESNO, timeout=-1, default=True)
	
	def resetLogfile(self,canstart=True):
		if not canstart:
			return
		print "[skyanytime] reset logfile."
		with open(self.sky_log_path , "w") as f:
			f.write('')
		self.changedEntry()
		self.readLog()
		
		
	def resetAdded(self,canstart=True):
		if not canstart:
			return
		sql.resetAdded()
		print "[skyanytime] reset table added."
		self.changedEntry()
		self.readAdded()
	
	
	def gotAnytimeFolder(self, res):
		if res is not None:
			config.plugins.skyanytime.anytimefolder.value = res
			config.plugins.skyanytime.anytimefolder.save()
			self.changedEntry()
			
	def gotSkydbFolder(self, res):
		if res is not None:
			config.plugins.skyanytime.skydb.value = res + "skydb.db"
			
			# check if we have to create the database from scratch
			res = True
			currentdb = getPluginPath() + "skydb.db"
			if os.path.exists(currentdb):
				try:
					shutil.copy2(currentdb, config.plugins.skyanytime.skydb.value)
				except:
					from SkyCreateDatabase import buildSkydb
					res = buildSkydb(target=config.plugins.skyanytime.skydb.value,rebuild=True,backup=False)
			else:
				from SkyCreateDatabase import buildSkydb
				res = buildSkydb(target=config.plugins.skyanytime.skydb.value,rebuild=True,backup=False)
			if res:
				config.plugins.skyanytime.skydb.save()
			self.changedEntry()
	
	def changedEntry(self):
		self.createConfigList()
		self["config"].setList(self.configlist)

	def saveSettings(self):
		print "saved"
		for x in self["config"].list:
			x[1].save()
		configfile.save()
		#config.save()
		self.close()
	
	def askUpdateDatabase(self):
		if config.plugins.skyanytime.auto_recordtimer_entries.value: 
			mymsg = "{0}\nSoll die Datenbank jetzt aktualisiert werden?\nAchtung, 'automatische Timer hinzufügen' ist aktiviert und wird im Anschluss ausgeführt. ".format(pluginName)
		else:
			mymsg = "{0}\nSoll die Datenbank jetzt aktualisiert werden?".format(pluginName)
		self.session.openWithCallback(self.updateDatabase,  MessageBox, _(mymsg), MessageBox.TYPE_YESNO, timeout=-1, default=True)
	
	
	def updateDatabase(self, canstart=True):
		if not canstart:
			return
		message = self.session.open(MessageBox, _("{0}\nAktualisiere Datenbank".format(pluginName)), MessageBox.TYPE_INFO, timeout=3)
		try:
			if SkyGetTvGuide.instance.refreshTimer:
				SkyGetTvGuide.instance.start(True)
			else:
				SkyGetTvGuide(self.session, True)
		except AttributeError:
			SkyGetTvGuide(self.session, True)

	
	def askAddRecordimerNow(self):
		mymsg = "{0}\nSollen alle automatischen Timer jetzt hinzugefügt werden?".format(pluginName)
		self.session.openWithCallback(self.addRecordimerNow,  MessageBox, _(mymsg), MessageBox.TYPE_YESNO, timeout=-1, default=True)
	
	
	def addRecordimerNow(self, canstart=True):
		if not canstart:
			return
		SkyRunAutocheck(self.session)
	
	
	
	def askCleanUpDatabase(self):
		self.includeAdded = False
		mymsg = "{0}\nSky TV-Guide Datenbank aufräumen.\nSollen auch alle gemerkten Timereinträge aus der Datenbank entfernt werden?".format(pluginName)
		self.session.openWithCallback(self.askCleanUpDatabaseGo,  MessageBox, _(mymsg), MessageBox.TYPE_YESNO, timeout=-1, default=False)
		
	
	def askCleanUpDatabaseGo(self,includeAdded=False):
		self.includeAdded = includeAdded
		mymsg = "{0}\nSky TV-Guide Datenbank jetzt leeren?".format(pluginName)
		self.session.openWithCallback(self.cleanUpDatabase,  MessageBox, _(mymsg), MessageBox.TYPE_YESNO, timeout=-1, default=False)
		
	def cleanUpDatabase(self, cleanUp=False):
		if cleanUp is not True:
			return

		res = sql.truncateDatabase(self.includeAdded)
		if res:
			self.session.open(MessageBox, _("{0}\nDie Datenbank wurde geleert.".format(pluginName)), MessageBox.TYPE_INFO, timeout=5)
		else:
			self.session.open(MessageBox, _("{0}\nDie Datenbank konnte nicht geleert werden.".format(pluginName)), MessageBox.TYPE_ERROR, timeout=-1)
		
	
	def keyCancel(self):
		for x in self["config"].list:
			x[1].cancel()
		self.close(False)
	
	def cancelSave(self):
		self.keyCancel()
